"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageProcessor = void 0;
const wechaty_1 = require("wechaty");
const bot_1 = require("../bot");
class MessageProcessor {
    constructor(context) {
        this.interceptors = [];
        this.events = {
            'error'(message, error) {
                console.warn(bot_1.template.use("on.error"));
                console.warn(error);
            }
        };
        this.context = context;
    }
    interceptor(interceptor) {
        return __awaiter(this, void 0, void 0, function* () {
            const usedName = this.interceptors.map(i => i.$name);
            const usedTitle = this.interceptors.map(i => i.$title);
            const usedAlias = [];
            this.interceptors.map(i => i.$alias).forEach(a => usedAlias.push(...a));
            const verifyExists = (str) => {
                if (usedName.includes(str))
                    throw new Error(`Cannot insert interceptor: ${str} already exists as an interceptor name`);
                if (usedTitle.includes(str))
                    throw new Error(`Cannot insert interceptor: ${str} already exists as an interceptor title`);
                if (usedAlias.includes(str))
                    throw new Error(`Cannot insert interceptor: ${str} already exists as an interceptor alias`);
            };
            verifyExists(interceptor.$name);
            verifyExists(interceptor.$title);
            if (interceptor.$alias && interceptor.$alias.length)
                interceptor.$alias.forEach(a => verifyExists(a));
            if (interceptor.$regist)
                yield interceptor.$regist(this.context);
            this.interceptors.push(interceptor);
        });
    }
    /**
     * 如果这个方法返回了文本，则将这个消息发送给来到时的会话
     * 否则，传递消息给下一个处理器
     */
    process(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const send = (content) => new Promise((resolve, reject) => {
                if (content === undefined ||
                    content === null ||
                    (content === "string" && content.length === 0))
                    resolve(void 0);
                else if (typeof content === "string")
                    message.say(content).then(resolve).catch(reject);
                else if (typeof content === "number")
                    message.say(content).then(resolve).catch(reject);
                else if (content instanceof wechaty_1.Contact)
                    message.say(content).then(resolve).catch(reject);
                else if (content instanceof wechaty_1.FileBox)
                    message.say(content).then(resolve).catch(reject);
                else if (content instanceof wechaty_1.UrlLink)
                    message.say(content).then(resolve).catch(reject);
                else if (content instanceof wechaty_1.MiniProgram)
                    message.say(content).then(resolve).catch(reject);
            });
            for (const i of this.interceptors) {
                if (!i)
                    continue;
                try {
                    let checkResult = true;
                    let checkerArgs = {};
                    for (const checker of i.$check) {
                        const result = yield checker(this.context, message, checkerArgs);
                        if (!result) {
                            checkResult = false;
                            break;
                        }
                        else if (typeof result === "object")
                            checkerArgs = Object.assign(Object.assign({}, checkerArgs), result);
                    }
                    if (!checkResult)
                        continue;
                    const sentMessages = [];
                    for (const handler of i.$handler) {
                        const result = yield handler(this.context, message, checkerArgs);
                        if (typeof result === "string" && !result.length)
                            continue;
                        if (result) {
                            const sentMessage = yield send(result);
                            sentMessages.push(sentMessage || undefined);
                        }
                    }
                    return sentMessages;
                }
                catch (e) {
                    const result = this.events.error(message, e);
                    const sentMessage = yield send(result);
                    return [sentMessage || undefined];
                }
            }
            return void 0;
        });
    }
    on(event, listener) {
        this.events[event] = listener;
    }
    usages(message) {
        return __awaiter(this, void 0, void 0, function* () {
            const results = [];
            for (const i of this.interceptors) {
                if (i.$usage)
                    results.push(`${i.$title}：${typeof i.$usage === "string" ? i.$usage : yield i.$usage(this.context, message)}`);
            }
            return results;
        });
    }
    usage(name, message) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const i of this.interceptors) {
                if (i.$name === name || i.$title === name || i.$alias.includes(name))
                    return `${i.$name}<br/>${typeof i.$usage === "string" ? i.$usage : yield i.$usage(this.context, message)}`;
            }
            return void 0;
        });
    }
    list() {
        return this.interceptors.map(i => {
            return { name: i.$name, title: i.$title, alias: i.$alias };
        });
    }
    attribute(name, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const i of this.interceptors) {
                if (i.$name === name || i.$title === name || i.$alias.includes(name))
                    return yield i.$attributes(this.context, ...args);
            }
            return void 0;
        });
    }
    attributes(...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = {};
            for (const i in this.interceptors) {
                result[this.interceptors[i].$name] = yield this.interceptors[i].$attributes(this.context, ...args);
            }
            return result;
        });
    }
}
exports.MessageProcessor = MessageProcessor;
